// ---------------------------------------------------------------------
//                            EXNAME
//
// This program illustrates how ekk_exportBasis can be used to write
// basis information for a problem that is initialized by ekk_addElementBlock.
//
// The program builds a model by reading a small MPS file to
// initialize the index control variables for row and column names.
// This file does not contain any model information.  It is used
// to force OSL to setup storage for row and column names.
//
// The problem data for the model is initialized by calling ekk_addElementBlock.
// Row and column names are initialized in the model by using the functions
// ekk_copyRowNames and ekk_copyColumnNames.
//
// Basis information from the resulting model can be written with 
// ekk_exportBasis and read with ekk_importBasis.  A model must have names 
// assigned to its rows and columns before ekk_exportBasis may be called.
// ---------------------------------------------------------------------

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

#define  MAXNC    2
#define  MAXNR    3
#define  MAXNEL   6
#define  MAXCHAR  8

int      mrow[MAXNEL]  = {0,0,1,1,2,2};
int      mcol[MAXNEL]  = {0,1,0,1,0,1};

double   dels[MAXNEL]  = {1.0,-1.0,-1.0,1.0,-1.0,-1.0};
double   cost[MAXNC]   = {-2.0,-6.0};
double   dlorow[MAXNR] = {-1.0,-1.0,-2.0};
double   duprow[MAXNR] = {1.0e+31,1.0e+31,1.0e+31};
double   dlocol[MAXNC] = {0.0,0.0};
double   dupcol[MAXNC] = {1.0e+31,1.0e+31};


int main()
{
    // Initialize OSL environment 
    EKKContext * env;
    env = ekk_initializeContext();
    
    // Create an OSL model "test" 
    EKKModel * test;
    test=ekk_newModel(env,NULL);

    // Read model data from MPS file 
    int rtcod = ekk_importModel(test,"exlp2.mps");

    // Set number of columns to 2 
    rtcod = ekk_setInumcols(test, 2);

    // Set number of rows to 3 
    rtcod = ekk_setInumrows(test, 3);

    // Set maximum allowable columns to 52 
    rtcod = ekk_setImaxcols(test, 2+50);

    // Set maximum allowable rows to 53 
    rtcod = ekk_setImaxrows(test, 3+50);

    // Set model's Column Bounds, names and Objective entries 
    int numcols = ekk_getInumcols(test);
    rtcod = ekk_copyCollower(test,dlocol,0,numcols);
    rtcod = ekk_copyColupper(test,dupcol,0,numcols);
    rtcod = ekk_copyObjective(test,cost,0,numcols);

    // Set model's Row Bounds and names 
    int numrows = ekk_getInumrows(test);
    rtcod = ekk_copyRowlower(test,dlorow,0,numrows);
    rtcod = ekk_copyRowupper(test,duprow,0,numrows);

#ifdef CREATE
    // Create model's Row/Column names 
    rtcod = ekk_createNames(test,MAXCHAR);
#else
    // Copy user specified Row/Column names to the model 
    char *colnames[] = {"C0000001","C0000002"};
    rtcod = ekk_copyColumnNames(test,colnames,0,numcols);

    char *rownames[]={"R0000001","R0000002","R0000003"};
    rtcod = ekk_copyRowNames(test,rownames,0,numrows);
#endif

    // Delete model's last block 
    rtcod = ekk_dropLastElementBlock(test);

    // Add a block of elements to model 
    rtcod = ekk_addElementBlock(test, MAXNEL, mrow, mcol, dels);

    // Merge all blocks in model into 1 block 
    rtcod = ekk_mergeBlocks(test,1);

    // Create a starting basis 
    rtcod = ekk_crash(test,2);

    // Write current basis to a file 
    rtcod = ekk_exportBasis(test, "basfile",1);

    // Solve using primal simplex method 
    rtcod = ekk_primalSimplex(test,1);

    // Print the solution 
    rtcod = ekk_printSolution(test);

    // Read in basis from a file 
    rtcod = ekk_importBasis(test, "basfile");

    // Solve using dual simplex method 
    rtcod = ekk_dualSimplex(test);

    // Print the solution 
    rtcod = ekk_printSolution(test);

    // Delete the model 
    rtcod = ekk_deleteModel(test);

    // OSL Destructor 
    ekk_endContext(env);

    return 0;
}
