// ---------------------------------------------------------------------
//                              EXROW
//
//   This program solves the following problem:
//
//   Maximize  4*x4 + 0.1*x5 + 6*x10 + 0.15*x11 + 8*x16 + 0.15*x17
//                  + 6*x22 + 0.15*x23 + 3*x26 + 0.1*x27
//                  + 3*x29 + 0.1*x30
//
//   Subject to Dx = b; x >= 0
//   where trans(b) = (0,0,0,0,0,0,0,0,0,0,100,50)
//   and the matrix D has the form:
//
//   ---------------
//   |      |  A   |
//   |  B   |      |
//   |      |------+-------
//   |      |      |  A   |
//   -------|  B   |      |
//          |      |------+-------
//          |      |      |  A   |
//          -------|  B   |      |
//                 |      |------+---------
//                 |      |      |        |
//                 -------|  B   |    C   |
//                        |      |        |
//                        |      |        |
//                        -----------------
//   --------
//   |  A   |
//   |      |
//   --------
//
//   Here A is a 2 X 6 matrix:
//
//            __                     __
//            | 1   1   1   1   0   0 |
//        A = |                       |
//            | 3   0   0   0   1   1 |
//            --                     --
//
//   B is a 4 X 6 matrix:
//
//            __                     __
//            | 0   0  -1   0   0   0 |
//            |                       |
//            | 0   0   0   0   0  -1 |
//        B = |                       |
//            |-4  -1   0   0   0   0 |
//            |                       |
//            | 0 -10   0   0   0   0 |
//            --                     --
//
//   C is a 4 X 6 matrix:
//
//            __                     __
//            | 1   1   0   0   0   0 |
//            |                       |
//            | 0   0   1   1   0   0 |
//        C = |                       |
//            |-1   0   0   0   1   0 |
//            |                       |
//            | 0   0   0  -1   0   1 |
//            --                     --
//
//   To illustrate the use of ekk_addOneRow and ekk_addColumns, this 
//   program first builds a matrix (D') which is D, but missing the last
//   occurrence of A and the occurrence of C.  The last A block is
//   added using ekk_addOneRow twice (once for each row) and C is added
//   using ekk_addColumns.
// --------------------------------------------------------------------- 

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
// This include is not really needed in this example; it would be if 
// the function fabs were used, say.
// Usually, it isn't a problem to include files that aren't needed,
// but this particular one causes a compile error (!) on the
// (old) HP we test on, so we have commented it out.
//#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

int main()
{
    // Initialize OSL environment 
    EKKContext * env = ekk_initializeContext();

    // Create an OSL model "test" 
    EKKModel * test = ekk_newModel(env,NULL);

    // Set to maximization problem   
    int rtcod = ekk_setMaximize(test);
   
    // ----------------------------------------------
    // Define Rim Data
    // ---------------------------------------------- 
    // Number of rows in D'
    const int NRows = 10; 
    // Number of cols in D"
    const int NCols = 24;     
    // Lower & Upper bounds on row activities   
    double rowRhs[NRows] = { 0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
    // Obj. func. coeff. on columns   
    double obj[NCols] = { 0.,0.,0.,4.,.1,0.,0.,0.,0.,6.,.15,0.,0.,0.,
        0.,8.,.15,0.,0.,0.,0.,6.,.15,0. };  
    // ----------------------------------------------

    // Set up the rim vectors of the model 
    rtcod = ekk_loadRimModel(
      test,NRows,rowRhs,rowRhs,NCols,obj,NULL,NULL);
 
    // ----------------------------------------------
    // Define "A" block as ordered triples
    // ---------------------------------------------- 
    // number of elements in A
    const int ANumElems = 7; 
    // Matrix elements - block A   
    double a[ANumElems]  = { 1.,3.,1.,1.,1.,1.,1.}; 
    // Row indices - block A   
    int    ia[ANumElems] = {0,1,0,0,0,1,1};
    // Column indices - block A   
    int    ja[ANumElems] = {0,0,1,2,3,4,5};
    // ----------------------------------------------

    // Set "A" block at position (0,6)  
    rtcod = ekk_addElementBlockWithOffsets(
      test,ANumElems,ia,ja,a,0,6);

    // Set "A" block at position (2,12)  
    rtcod = ekk_addElementBlockWithOffsets(
      test,ANumElems,ia,ja,a,2,12);

    // Set "A" block at position (4,18)  
    rtcod = ekk_addElementBlockWithOffsets(
      test,ANumElems,ia,ja,a,4,18);
 
    // ----------------------------------------------
    // Define "B" block as storage by columns
    // ----------------------------------------------
    // number of elements in B   
    const int BNumElems = 5; 
    // number of columns in B  
    const int BNumCols = 6; 
    // Matrix elements - block B    
    double b[BNumElems]  = { -4.,-1.,-10.,-1.,-1.}; 
    // Row indices - block B   
    int    ib[BNumElems] = {2,2,3,0,1};
    // Column starts - block B   
    int    jb[BNumCols+1] = {0,1,3,4,4,4,5};     
    // ----------------------------------------------

    // Set "B" block at position (0,0)  
    rtcod = ekk_addColumnElementBlock(test,BNumCols,ib,jb,b);

    // Set "B" block at position (2,6)  
    rtcod = ekk_addColumnElementBlockWithOffsets(
      test,BNumCols,6,2,ib,jb,b);

    // Set "B" block at position (4,12)  
    rtcod = ekk_addColumnElementBlockWithOffsets(
      test,BNumCols,12,4,ib,jb,b);

    // Set "B" block at position (6,18)  
    rtcod = ekk_addColumnElementBlockWithOffsets(
      test,BNumCols,18,6,ib,jb,b);
    
    // ----------------------------------------------
    // This complete the building of D'.
    // The last A block and the C block remain to be added to the model.
    // The A block is now added by adding a 10 and 11th row to the model.   
    // ----------------------------------------------
 
    // ----------------------------------------------
    // Define first row "A" block
    // ----------------------------------------------
    // number of elements in row 0 of A  
    const int ARow0NumElems = 4; 
    // Elements in row 0 of A
    const double  ar0Elems[ARow0NumElems] = {1.0,1.0,1.0,1.0};
    // Indices of elements in row 0 of A
    const int  ar0Indices[ARow0NumElems] = {0,1,2,3};
    // ----------------------------------------------

    // Add first row of A to the model
    ekk_addOneRow(test, 100., 100, ARow0NumElems, ar0Indices, ar0Elems );
    
    // ----------------------------------------------
    // Define second row "A" block
    // ----------------------------------------------
    // number of elements in row 1 of A  
    const int ARow1NumElems = 3; 
    // Elements in row 1 of A
    const double  ar1Elems[ARow1NumElems] = {3.0,1.0,1.0};
    // Indices of elements in row 1 of A
    const int  ar1Indices[ARow1NumElems] = {0,4,5};
    // ----------------------------------------------

    // Add second row of A to the model
    ekk_addOneRow(test, 50., 50, ARow1NumElems, ar1Indices, ar1Elems ); 
    
    // ----------------------------------------------
    // The C block remains to be added to the model.  
    // ----------------------------------------------

    // ----------------------------------------------
    // Define "C" block in storage by columns format
    // ----------------------------------------------  
    // number of elements in C  
    const int CNumElems = 8;    
    // number of columns in C  
    const int CNumCols = 6; 
    // Matrix elements - block C   
    double c[CNumElems]  = { 1.,-1.,1.,1.,1.,-1.,1.,1. };
    // Row indices - block C   
    int    ic[CNumElems] = { 0+6,2+6,0+6,1+6,1+6,3+6,2+6,3+6 };
    // Column starts - block C   
    int    jc[CNumCols+1] = { 0,2,3,4,6,7,8 };
    // Objective Coefficients for block C columns
    double objCofCols24to29[CNumCols] = { 0., 3., .1, 0., 3., .1 };
    // ----------------------------------------------

    // Add block C columns to the model
    ekk_addColumns(test, CNumCols, objCofCols24to29, NULL, NULL,
      jc, ic, c );

    // Consolidate all the block sinto one block 
    rtcod = ekk_mergeBlocks(test,1);

    // Solve using simplex method 
    rtcod = ekk_primalSimplex(test,2);

    // Set to print the nonzero column solutions only 
    rtcod = ekk_setIsolmask(test,6);

    // Print the solution 
    rtcod = ekk_printSolution(test);

    // Delete the model 
    rtcod = ekk_deleteModel(test);

    // OSL Destructor 
    ekk_endContext(env);

    return 0;
}
